==> fl_camera-1.1.1.tar.gz: lib/src/fl_camera.dart <==
part of '../fl_camera.dart';

class FlCamera extends StatefulWidget {
  const FlCamera(
      {Key? key,
      this.overlay,
      this.uninitialized,
      this.onFlashChange,
      this.onZoomChange,
      this.updateReset = false,
      this.camera,
      this.resolution = CameraResolution.high,
      this.fit = BoxFit.fitWidth})
      : super(key: key);

  /// 显示在预览框上面
  /// Display above preview box
  final Widget? overlay;

  /// 相机在未初始化时显示的UI
  /// The UI displayed when the camera is not initialized
  final Widget? uninitialized;

  /// 闪光灯变化
  /// Flash change
  final ValueChanged<FlashState>? onFlashChange;

  /// 缩放变化
  /// zoom ratio
  final ValueChanged<CameraZoomState>? onZoomChange;

  /// 更新组件时是否重置相机
  /// Reset camera when updating components
  final bool updateReset;

  /// 需要预览的相机
  /// Camera ID to preview
  final CameraInfo? camera;

  /// 预览相机支持的分辨率
  /// Preview the resolution supported by the camera
  final CameraResolution resolution;
  final BoxFit fit;

  @override
  _FlCameraStateWidget createState() => _FlCameraStateWidget();
}

class _FlCameraStateWidget extends FlCameraState<FlCamera> {
  @override
  void initState() {
    currentChannel = _flCameraChannel;
    super.initState();
    WidgetsBinding.instance!.addPostFrameCallback((Duration time) => init());
  }

  Future<void> init() async {
    boxFit = widget.fit;

    /// 添加消息回调
    /// Add message callback
    await initEvent(eventListen);

    /// 初始化相机
    /// Initialize camera
    if (await initCamera(camera: widget.camera, resolution: widget.resolution))
      setState(() {});
  }

  @override
  void onFlashChange(FlashState state) {
    super.onFlashChange(state);
    if (widget.onFlashChange != null) widget.onFlashChange!(state);
  }

  @override
  void onZoomChange(CameraZoomState zoomState) {
    super.onZoomChange(zoomState);
    if (widget.onZoomChange != null) widget.onZoomChange!(zoomState);
  }

  void eventListen(dynamic data) {
    print('Received message from native $data');
  }

  @override
  void didUpdateWidget(covariant FlCamera oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (oldWidget.overlay != widget.overlay ||
        oldWidget.onFlashChange != widget.onFlashChange ||
        oldWidget.uninitialized != widget.uninitialized) {
      if (widget.updateReset)
        cameraMethodCall.dispose().then((bool value) {
          if (value) init();
        });
    }
  }

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    if (state == AppLifecycleState.resumed) {
      init();
    } else {
      super.didChangeAppLifecycleState(state);
    }
  }

  @override
  Widget build(BuildContext context) {
    Widget camera = super.build(context);
    if (widget.overlay != null)
      camera = Stack(children: <Widget>[
        camera,
        SizedBox.expand(child: widget.overlay),
      ]);
    return camera;
  }
}

abstract class FlCameraState<T extends StatefulWidget> extends State<T>
    with WidgetsBindingObserver {
  /// 相机方法
  /// Camera method
  late FlCameraMethodCall cameraMethodCall;

  /// 当前使用的channel
  /// Currently used channel
  MethodChannel? currentChannel;

  /// 相机在未初始化时显示的UI
  /// The UI displayed when the camera is not initialized
  Widget? uninitialized;

  /// 是否全屏 全屏会拉伸相机画面
  /// fullScreen
  bool fullScreen = false;

  /// camera fit
  BoxFit? boxFit;

  @override
  void initState() {
    super.initState();
    assert(currentChannel != null,
        '"currentchannel" must be initialized before super.Initstate()');
    WidgetsBinding.instance!.addObserver(this);
    initOptions();
  }

  void initOptions() {
    cameraMethodCall = FlCameraMethodCall.instance;
    cameraMethodCall.setMethodChannel(currentChannel!);
    WidgetsBinding.instance!.addPostFrameCallback((Duration duration) async {
      await SystemChrome.setPreferredOrientations(
          <DeviceOrientation>[DeviceOrientation.portraitUp]);
      SystemChrome.setSystemUIOverlayStyle(SystemUiOverlayStyle.light
          .copyWith(statusBarColor: Colors.transparent));
    });
  }

  /// 初始化相机
  /// Initialize camera
  /// [camerId] 需要预览的相机 Camera ID to preview
  /// [resolution] 预览相机支持的分辨率 Preview the resolution supported by the camera
  Future<bool> initCamera(
      {CameraInfo? camera,
      CameraResolution resolution = CameraResolution.high}) async {
    if (camera == null) {
      final List<CameraInfo>? cameras =
          await cameraMethodCall.availableCameras();
      if (cameras == null) return false;
      for (final CameraInfo cameraInfo in cameras) {
        if (cameraInfo.lensFacing == CameraLensFacing.back) {
          camera = cameraInfo;
          break;
        }
      }
    }
    if (camera == null) return false;

    bool state = await cameraMethodCall.initCamera();
    if (state) {
      final FlCameraOptions? previewInfo = await cameraMethodCall
          .startPreview(camera.name, resolution: resolution);
      state = previewInfo != null;
    }
    return state;
  }

  /// 初始化消息通道
  /// Initialize message channel
  Future<bool> initEvent(EventListen eventListen) async {
    final bool state = await cameraMethodCall.cameraEvent.initialize();
    if (state)
      cameraMethodCall.cameraEvent
          .addListener((dynamic data) => _eventListen(eventListen, data));
    return state;
  }

  void _eventListen(EventListen? eventListen, dynamic data) {
    if (eventListen != null) eventListen(data);
    if (data is Map) {
      /// zoomRatio
      final double? zoomRatio = data['zoomRatio'] as double?;
      if (zoomRatio != null) {
        final double? maxZoomRatio = data['maxZoomRatio'] as double?;
        if (maxZoomRatio != null) {
          onZoomChange(CameraZoomState(
              maxZoomRatio: maxZoomRatio, zoomRatio: zoomRatio));
        }
      }

      /// flash state
      final bool? flash = data['flash'] as bool?;
      if (flash != null) {
        FlashState flashState = FlashState.auto;
        flashState = flash ? FlashState.on : FlashState.off;
        onFlashChange(flashState);
      }
    }
  }

  /// 缩放变化
  /// zoom ratio
  void onZoomChange(CameraZoomState state) {}

  /// 闪光灯变化
  /// flash change
  void onFlashChange(FlashState state) {}

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    if (state != AppLifecycleState.resumed) cameraMethodCall.dispose();
  }

  @override
  Widget build(BuildContext context) {
    boxFit ??= fullScreen ? BoxFit.fitHeight : BoxFit.fitWidth;
    Widget current = const SizedBox();
    final FlCameraOptions? cameraOptions = cameraMethodCall.cameraOptions;
    if (cameraOptions != null && cameraOptions.textureId != null) {
      current = ColoredBox(
          color: Colors.black,
          child: FittedBox(
              fit: boxFit!,
              child: SizedBox.fromSize(
                  size: Size(cameraOptions.width!, cameraOptions.height!),
                  child: Texture(textureId: cameraOptions.textureId!))));
    } else {
      current = uninitialized ??
          Container(
              color: Colors.black,
              alignment: Alignment.center,
              child: const Text('Camera not initialized',
                  style: TextStyle(color: Colors.white)));
    }
    return SizedBox.expand(child: current);
  }

  @override
  void dispose() {
    cameraMethodCall.dispose();
    super.dispose();
    WidgetsBinding.instance!.removeObserver(this);
  }
}

class CameraZoomState {
  CameraZoomState({this.maxZoomRatio, this.zoomRatio});

  CameraZoomState.fromMap(Map<String, dynamic> map)
      : maxZoomRatio = map['maxZoomRatio'] as double?,
        zoomRatio = map['maxZoomRatio'] as double?;

  /// 最大缩放比例
  /// Camera max zoom ratio
  double? maxZoomRatio;

  /// 当前缩放比例
  /// Current zoom ratio
  double? zoomRatio;
}

==> fl_camera-1.2.0.tar.gz: lib/src/fl_camera.dart <==
part of '../fl_camera.dart';

class FlCamera extends StatefulWidget {
  const FlCamera(
      {Key? key,
      this.overlay,
      this.uninitialized,
      this.onFlashChange,
      this.onZoomChange,
      this.updateReset = false,
      this.camera,
      this.resolution = CameraResolution.high,
      this.fit = BoxFit.fitWidth})
      : super(key: key);

  /// 显示在预览框上面
  /// Display above preview box
  final Widget? overlay;

  /// 相机在未初始化时显示的UI
  /// The UI displayed when the camera is not initialized
  final Widget? uninitialized;

  /// 闪光灯变化
  /// Flash change
  final ValueChanged<FlashState>? onFlashChange;

  /// 缩放变化
  /// zoom ratio
  final ValueChanged<CameraZoomState>? onZoomChange;

  /// 更新组件时是否重置相机
  /// Reset camera when updating components
  final bool updateReset;

  /// 需要预览的相机
  /// Camera ID to preview
  final CameraInfo? camera;

  /// 预览相机支持的分辨率
  /// Preview the resolution supported by the camera
  final CameraResolution resolution;
  final BoxFit fit;

  @override
  _FlCameraStateWidget createState() => _FlCameraStateWidget();
}

class _FlCameraStateWidget extends FlCameraState<FlCamera> {
  @override
  void initState() {
    currentChannel = _flCameraChannel;
    super.initState();
    WidgetsBinding.instance!.addPostFrameCallback((Duration time) => init());
  }

  Future<void> init() async {
    boxFit = widget.fit;

    /// 添加消息回调
    /// Add message callback
    await initEvent(eventListen);

    /// 初始化相机
    /// Initialize camera
    if (await initCamera(camera: widget.camera, resolution: widget.resolution))
      setState(() {});
  }

  @override
  void onFlashChange(FlashState state) {
    super.onFlashChange(state);
    if (widget.onFlashChange != null) widget.onFlashChange!(state);
  }

  @override
  void onZoomChange(CameraZoomState zoomState) {
    super.onZoomChange(zoomState);
    if (widget.onZoomChange != null) widget.onZoomChange!(zoomState);
  }

  void eventListen(dynamic data) {
    print('Received message from native $data');
  }

  @override
  void didUpdateWidget(covariant FlCamera oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (oldWidget.overlay != widget.overlay ||
        oldWidget.onFlashChange != widget.onFlashChange ||
        oldWidget.uninitialized != widget.uninitialized) {
      if (widget.updateReset)
        cameraMethodCall.dispose().then((bool value) {
          if (value) init();
        });
    }
  }

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    if (state == AppLifecycleState.resumed) {
      init();
    } else {
      super.didChangeAppLifecycleState(state);
    }
  }

  @override
  Widget build(BuildContext context) {
    Widget camera = super.build(context);
    if (widget.overlay != null)
      camera = Stack(children: <Widget>[
        camera,
        SizedBox.expand(child: widget.overlay),
      ]);
    return camera;
  }
}

abstract class FlCameraState<T extends StatefulWidget> extends State<T>
    with WidgetsBindingObserver {
  /// 相机方法
  /// Camera method
  late FlCameraMethodCall cameraMethodCall;

  /// 当前使用的channel
  /// Currently used channel
  MethodChannel? currentChannel;

  /// 相机在未初始化时显示的UI
  /// The UI displayed when the camera is not initialized
  Widget? uninitialized;

  /// 是否全屏 全屏会拉伸相机画面
  /// fullScreen
  bool fullScreen = false;

  /// camera fit
  BoxFit? boxFit;

  @override
  void initState() {
    super.initState();
    assert(currentChannel != null,
        '"currentchannel" must be initialized before super.Initstate()');
    WidgetsBinding.instance!.addObserver(this);
    initOptions();
  }

  void initOptions() {
    cameraMethodCall = FlCameraMethodCall();
    cameraMethodCall.setMethodChannel(currentChannel!);
    WidgetsBinding.instance!.addPostFrameCallback((Duration duration) async {
      await SystemChrome.setPreferredOrientations(
          <DeviceOrientation>[DeviceOrientation.portraitUp]);
      SystemChrome.setSystemUIOverlayStyle(SystemUiOverlayStyle.light
          .copyWith(statusBarColor: Colors.transparent));
    });
  }

  /// 初始化相机
  /// Initialize camera
  /// [camerId] 需要预览的相机 Camera ID to preview
  /// [resolution] 预览相机支持的分辨率 Preview the resolution supported by the camera
  Future<bool> initCamera(
      {CameraInfo? camera,
      CameraResolution resolution = CameraResolution.high}) async {
    if (camera == null) {
      final List<CameraInfo>? cameras =
          await cameraMethodCall.availableCameras();
      if (cameras == null) return false;
      for (final CameraInfo cameraInfo in cameras) {
        if (cameraInfo.lensFacing == CameraLensFacing.back) {
          camera = cameraInfo;
          break;
        }
      }
    }
    if (camera == null) return false;

    bool state = await cameraMethodCall.initCamera();
    if (state) {
      final FlCameraOptions? previewInfo = await cameraMethodCall
          .startPreview(camera.name, resolution: resolution);
      state = previewInfo != null;
    }
    return state;
  }

  /// 初始化消息通道
  /// Initialize message channel
  Future<bool> initEvent(EventListen eventListen) async {
    final bool state = await cameraMethodCall.cameraEvent.initialize();
    if (state)
      cameraMethodCall.cameraEvent
          .addListener((dynamic data) => _eventListen(eventListen, data));
    return state;
  }

  void _eventListen(EventListen? eventListen, dynamic data) {
    if (eventListen != null) eventListen(data);
    if (data is Map) {
      /// zoomRatio
      final double? zoomRatio = data['zoomRatio'] as double?;
      if (zoomRatio != null) {
        final double? maxZoomRatio = data['maxZoomRatio'] as double?;
        if (maxZoomRatio != null) {
          onZoomChange(CameraZoomState(
              maxZoomRatio: maxZoomRatio, zoomRatio: zoomRatio));
        }
      }

      /// flash state
      final bool? flash = data['flash'] as bool?;
      if (flash != null) {
        FlashState flashState = FlashState.auto;
        flashState = flash ? FlashState.on : FlashState.off;
        onFlashChange(flashState);
      }
    }
  }

  /// 缩放变化
  /// zoom ratio
  void onZoomChange(CameraZoomState state) {}

  /// 闪光灯变化
  /// flash change
  void onFlashChange(FlashState state) {}

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    if (state != AppLifecycleState.resumed) cameraMethodCall.dispose();
  }

  @override
  Widget build(BuildContext context) {
    boxFit ??= fullScreen ? BoxFit.fitHeight : BoxFit.fitWidth;
    Widget current = const SizedBox();
    final FlCameraOptions? cameraOptions = cameraMethodCall.cameraOptions;
    if (cameraOptions != null && cameraOptions.textureId != null) {
      current = ColoredBox(
          color: Colors.black,
          child: FittedBox(
              fit: boxFit!,
              child: SizedBox.fromSize(
                  size: Size(cameraOptions.width!, cameraOptions.height!),
                  child: Texture(textureId: cameraOptions.textureId!))));
    } else {
      current = uninitialized ??
          Container(
              color: Colors.black,
              alignment: Alignment.center,
              child: const Text('Camera not initialized',
                  style: TextStyle(color: Colors.white)));
    }
    return SizedBox.expand(child: current);
  }

  @override
  void dispose() {
    cameraMethodCall.dispose();
    super.dispose();
    WidgetsBinding.instance!.removeObserver(this);
  }
}

class CameraZoomState {
  CameraZoomState({this.maxZoomRatio, this.zoomRatio});

  CameraZoomState.fromMap(Map<String, dynamic> map)
      : maxZoomRatio = map['maxZoomRatio'] as double?,
        zoomRatio = map['maxZoomRatio'] as double?;

  /// 最大缩放比例
  /// Camera max zoom ratio
  double? maxZoomRatio;

  /// 当前缩放比例
  /// Current zoom ratio
  double? zoomRatio;
}
